VampArrow = LibStub("AceAddon-3.0"):NewAddon("VampArrow", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0", "AceComm-3.0", "AceSerializer-3.0")
VampArrow.MINOR_VERSION = tonumber(("$Revision: 15 $"):match("%d+"))
local L = LibStub("AceLocale-3.0"):GetLocale("VampArrow")
local VampArrow = VampArrow
----


--[[
      VARIABLES ----------------------------------------------------------------------------------------------------------------
  ]]

local isEnabled = false
local POLLING_FREQUENCY = 1.0
local raid = {}
local assignment
local assignmentTime
local vampireDebuff = GetSpellInfo(70867) -- Essence of the Blood Queen
local vampireFrenzyDebuff = GetSpellInfo(71474) -- Frenzied Bloodthirst
local mindControlledDebuff = GetSpellInfo(70923) -- Uncontrollable Frenzy
local testDebuff
--[===[@debug@
testDebuff = 58600 -- Restricted Flight Area
--@end-debug@]===]
VampArrow.inCombat = false
local commPrefix = "VampArrow-1.0"
local arrowMoving = false
local scene



--[[
      UPVALUES -----------------------------------------------------------------------------------------------------------------
  ]]
  
local _G = getfenv(0)
local pairs = _G.pairs
local strlower = _G.strlower
local strupper = _G.strupper
local format = _G.format



--[[
      UTILITIES ----------------------------------------------------------------------------------------------------------------
  ]]

function VampArrow:tcopy(to, from)
 for k, v in pairs(from) do
  if type(v) == "table" then
   to[k] = {}
   VampArrow:tcopy(to[k], v)
  else
   to[k] = v
  end
 end
end


local newOrder
do
	local current = 0
	function newOrder()
		current = current + 1
		return current
	end
end


local function colorize(s, color, highlight)
	if color and s then
		if highlight then
			return string.format("|cff%02x%02x%02x%s|r", (color.r or 1) * 128 + 127, (color.g or 1) * 128 + 127, (color.b or 1) * 128 + 127, s)
		else
			return string.format("|cff%02x%02x%02x%s|r", (color.r or 1) * 255, (color.g or 1) * 255, (color.b or 1) * 255, s)
		end
	else
		return s
	end
end



--[[
      OPTIONS ------------------------------------------------------------------------------------------------------------------
  ]]

VampArrow.options = {
	name = "VampArrow         r|cff20ff20"..VampArrow.MINOR_VERSION.."|r",
	handler = VampArrow,
	type = "group",
	childGroups = "tab",
	args = {		

		enableArrow = {
			type = "toggle",
			name = L["Enable Arrow"],
			desc = L["Enables the arrow that points toward your vampire target."],
			get = function() return VampArrow.db.profile.enableArrow end,
			set = function(self, val) VampArrow.db.profile.enableArrow = val end,
			order = newOrder(),
		}, -- enableArrow

		enableAvr = {
			type = "toggle",
			name = L["Enable AVR Integration"],
			desc = L["If AVR is installed, shows a circle under your vampire target."],
			get = function() return VampArrow.db.profile.enableAvr end,
			set = function(self, val) VampArrow.db.profile.enableAvr = val end,
			order = newOrder(),
		}, -- enableAvr

		toggleArrow = {
			type = "execute",
			name = L["Toggle Arrow"],
			desc = L["Shows or hides the arrow, allowing you to move it."],
			func = function(self) VampArrow:ToggleArrow() end,
			order = newOrder(),
		}, -- toggleArrow

		lockArrow = {
			type = "toggle",
			name = L["Lock Arrow"],
			desc = L["Prevents the arrow from moving when you drag it."],
			get = function() return VampArrow.db.profile.lockArrow end,
			set = function(self, val) VampArrow.db.profile.lockArrow = val end,
			order = newOrder(),
		}, -- lockArrow

	}
}


function VampArrow:ToggleArrow()
	if arrowMoving then
		arrowMoving = false
		VampArrow.Arrow:EndMove()
	else
		arrowMoving = true
		VampArrow.Arrow:Move()
	end
end



--[[
      INITIALIZATION -----------------------------------------------------------------------------------------------------------
  ]]

function VampArrow:OnInitialize()
	-- Register database
 self.db = LibStub("AceDB-3.0"):New("VampArrowDB", self.defaults, "Default")
	self.db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileDeleted", "OnProfileChanged")
		
	-- Register options
	LibStub("AceConfig-3.0"):RegisterOptionsTable("VampArrow", VampArrow.options)
	VampArrow.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("VampArrow", "VampArrow")
	LibStub("AceConfig-3.0"):RegisterOptionsTable("VampArrow-Profiles", LibStub("AceDBOptions-3.0"):GetOptionsTable(VampArrow.db))
	VampArrow.profilesFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("VampArrow-Profiles", "Profiles", "VampArrow")

	-- Register for slash commands
	self:RegisterChatCommand("vamparrow", "ChatCommand")
	self:RegisterChatCommand("va", "ChatCommand")
	
	-- Map scale
	VampArrow:RegisterMapSize("IcecrownCitadel",
		1, 1355.47009278, 903.647033691,	-- The Lower Citadel
		2, 1067, 711.3336906438,			      -- The Rampart of Skulls
		3, 195.46997071, 130.315002441,		-- Deathbringer's Rise
		4, 773.71008301, 515.81030273,	 	-- The Frost Queen's Lair
		5, 1148.73999024, 765.82006836,		-- The Upper Reaches
		6, 373.70996094, 249.12988281,		 -- Royal Quarters
		7, 293.26000977, 195.507019042,		-- The Frozen Throne
		8, 247.92993165, 165.287994385	 	-- Frostmourne
	)
	--[===[@debug@
	VampArrow:RegisterMapSize("Dalaran", 0, 0.0, 0.0)
	VampArrow:RegisterMapSize("Moonglade", 0, 2308.33325195313, 1539.5830078125)
	--@end-debug@]===]
end


function VampArrow:OnEnable()
	-- Register for events
	self:UnregisterAllEvents()
	self:RegisterEvent("RAID_ROSTER_UPDATE", "OnEvent")
	self:RegisterEvent("PLAYER_REGEN_DISABLED", "OnEvent")
	self:RegisterEvent("PLAYER_REGEN_ENABLED", "OnEvent")
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED", "OnCombatEvent")

	-- Register communication
	self:UnregisterAllComm()
 self:RegisterComm(commPrefix, "OnCommReceived")
 
	-- Update data
	self:UpdateRaid()
	
	-- Initialize arrow
	VampArrow.Arrow:LoadPosition()
	self:ClearTarget()

	-- Turn us on
	isEnabled = true

	--[===[@debug@
	self:Print(L["Loaded"])
	--@end-debug@]===]
	
end


function VampArrow:OnDisable()
	-- Cancel all timers
	self:CancelAllTimers()

	-- Turn us off
	isEnabled = false
	
	--[===[@debug@
	self:Print(L["Disabled"])
	--@end-debug@]===]
end


function VampArrow:ChatCommand(input)
	InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
end


function VampArrow:OnProfileChanged(event, database, newProfileKey)
	--[===[@debug@
	self:Print(L["Profile modified, rebooting"])
	--@end-debug@]===]
	
 self:Disable()
 self:Enable() 
end



--[[
      EVENTS AND FIGHT LOGIC ---------------------------------------------------------------------------------------------------
  ]]
  
function VampArrow:OnCommReceived(prefix, message, distribution, sender)
 local success, o = self:Deserialize(message)
 if success == false then
		--[===[@debug@
  self:Print("[OnCommReceived] Failed to deserialize message")
		--@end-debug@]===]
  return
 else
		if o.c then
		
			if o.c == "SET" then
				if o.t then
					self:SetTarget(o.t, o.a or false, o.r or false, o.m, o.dm or false)
				end
				
			elseif o.c == "VER_REQ" then
				self:SendCommMessage(commPrefix, self:Serialize({ c = "VER", v = self.MINOR_VERSION }), "WHISPER", sender)
				
			elseif o.c == "CLEAR" then
				self:ClearTarget()
				
			elseif o.c == "DISABLE" then
				if isEnabled then self:Disable() end
				
			elseif o.c == "ENABLE" then
				if not isEnabled then self:Enable() end
				
			end
			
		end
 end
end


function VampArrow:OnEvent(event, ...)

	if event == "RAID_ROSTER_UPDATE" then
		self:UpdateRaid()
						
	end


 if isEnabled then

		if event == "PLAYER_REGEN_DISABLED" then
			self.inCombat = true
			if UnitInRaid("player") then
				self:ResetFight()
				self:ScheduleRepeatingTimer("OnTimer", POLLING_FREQUENCY)
			end
			
		elseif event == "PLAYER_REGEN_ENABLED" then
			self.inCombat = false
			self:CancelAllTimers()
			self:UpdateRaid()
			self:ClearTarget()
			
		end
		
 end
end


function VampArrow:UpdateRaid()
	if UnitInRaid("player") then
		if not self.inCombat then raid = {} end
		local numRaid = GetNumRaidMembers()
		for i = 1, numRaid do
			local playerName, rank, subgroup, level, class, fileName, zone, online, isDead, role, isML = GetRaidRosterInfo(i)
			if playerName then
				if not self.inCombat then
					raid[playerName] = {}
					raid[playerName].group = subgroup
					raid[playerName].rank = rank
				else
					if not raid[playerName] then raid[playerName] = {} end
					raid[playerName].group = subgroup
					raid[playerName].rank = rank

					if not online or isDead then
						-- Someone died or disconnected during combat. Let's see if it's someone we care about
						if playerName == UnitName("player") or (assignment and playerName == assignment) then
							self:ClearTarget()
						end
					end -- disconnected player
				end -- in combat
			end -- playerName not nil
		end -- player loop
	end -- we're in a raid
end



function VampArrow:ResetFight()
	self:ClearTarget()
end


function VampArrow:ClearTarget()
	assignment = nil
	assignmentTime = nil
	VampArrow.Arrow.mark = nil
	VampArrow.Arrow:EndMove()
	if scene then
		scene:ClearScene()
		scene.visible = false
	end
end


function VampArrow:SetTarget(target, announce, reverse, mark, doMark)
	if isEnabled then
		if not announce then announce = false end
		if not reverse then reverse = false end
		
		assignment = target
		assignmentTime = GetTime()
		
		VampArrow.Arrow.mark = nil
		if mark and doMark then
			VampArrow.Arrow.mark = mark
		end
		
		if self.db.profile.enableArrow then
			VampArrow.Arrow:ShowRunTo(target)
		end
		
		if self.db.profile.enableAvr and AVR then
			if not scene then
				scene = AVR:GetTempScene("VAMPARROW_AVR")
			end
			scene:ClearScene()
			scene.visible = true
			--AVR:AddCircleWarning(scene, target, true, 2, nil, .2, 1, .2, .5)
			local m = AVRMarkerMesh:New()
			m:SetFollowUnit(target)
			m:SetLine(true)
			m:SetClassColor(true)
			m:SetColor(1.0, 1.0, 1.0, 0.8)
			scene:AddMesh(m)
		end
		
		if target and announce then
			if reverse then
				SendChatMessage(format(L["Biting %s!"], target), "YELL")
			else
				SendChatMessage(format(L["%s is biting me!"], target), "YELL")
			end
		end
	end
end


function VampArrow:OnTimer()
	self:UpdateRaid()
end


function VampArrow:OnCombatEvent(event, timestamp, eventType, srcGuid, srcName, srcFlags, dstGuid, dstName, dstFlags, ...)
	if isEnabled and UnitInRaid("player") then
	
------[[ AURAS ]] ---------------------------------------------------------------------------------------------------------------------
		if eventType == "SPELL_AURA_APPLIED" or eventType == "SPELL_AURA_REFRESH" then
		
			-- We only care about ourself or our assignment
			if (srcName == UnitName("player") or dstName == UnitName("player")) or (assignment and (srcName == assignment or dstName == assignment)) then
				local spellId, spellName, spellSchool, auraType = select(1, ...)
				
				---------------------------- Player was just bit or successfully bit another player
				if spellName == vampireDebuff or spellId == testDebuff then
				
					self:ClearTarget()
		
		
				---------------------------- Player is in frenzy mode, they must bite someone
				elseif spellName == vampireFrenzyDebuff then

					-- nothing

				
				---------------------------- Player was MCed
				elseif spellName == mindControlledDebuff then
				
					self:ClearTarget()


				end -- spell
			end -- player is someone we care about
		end -- eventType
	end -- addon enabled, etc.
end

